package no.ntnu.stud.frodesol.jexserver.bin;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import no.ntnu.stud.frodesol.jexserver.*;
import no.ntnu.stud.frodesol.jexserver.eval.*;
import no.ntnu.stud.frodesol.jexserver.util.SocketFileUtil;
import no.ntnu.stud.frodesol.jexserver.core.JexEnv;
import no.ntnu.stud.frodesol.jexserver.dispatch.*;


public class EvalClient {
    
    static final int BUFFER_SIZE = 65536;
    
    /**
     * @param args
     */
    public static void main (String[] args) {
        
        System.out.println ("JExercise evaluator client starting");
        
        while (true) {
            
            EvalClient client = null;
            try {
                client = new EvalClient ();
                if (args.length > 0) {
                    client.run (args[0]);
                }
                else {
                    client.run ();
                }
            }
            catch (Exception e) {
                System.out.println ("Exception in EvalClient main: " + e);
                e.printStackTrace ();
                try {
                    System.out.println ("Sleeping for 5 seconds");
                    Thread.sleep (5000);
                }
                catch (InterruptedException f) {}
            }
        }
    }
    
    private String m_tempdir;
    
    public EvalClient () throws IOException {
        
    }

    private void run (String host) throws Exception {
        System.out.println ("EvalClient.run ()");
        
        System.out.println("Connecting to " + host);
        //String host = JexEnv.getDispatcherHost ();
        int port = JexEnv.getDispatcherPort ();
        
        Socket socket = new Socket ();
        SocketAddress sockaddr = new InetSocketAddress (host, port);
        socket.connect (sockaddr, 10000);
        System.out.println ("Socket connected");
        
        InputStream is = socket.getInputStream ();
        OutputStream os = socket.getOutputStream ();
        
        while (true) {
            processOne (is, os);
        }
    }
    
    private void run () throws Exception {
        String host = JexEnv.getDispatcherHost ();
        run (host);
    }
    
    private void processOne (InputStream is, OutputStream os)
            throws Exception {
        
        UUID uuid = UUID.randomUUID ();
        File file = new File (JexEnv.getEvalTemp (), uuid.toString ());
        m_tempdir = file.getPath ();
        
        try {
            file.mkdirs ();
            // Fetch, process, and report results
            readExercise (is);
            readSubmission (is);
            unzipTests ();
            try {
                unzipSubmission ();
            }
            catch (Exception e) {
                writeErrorReport("Could not unzip submission: " + 
                        e.getMessage());
                writeResults (os);
                return;
            }
            runEvaluator ();
            writeResults (os);
        }
        finally {
            no.ntnu.stud.frodesol.jexserver.util.Util.deleteFolder (m_tempdir);
        }
    }
    
    private void runEvaluator () {
        
        no.ntnu.stud.frodesol.jexserver.eval.Evaluator evaluator = 
            new no.ntnu.stud.frodesol.jexserver.eval.Evaluator (m_tempdir);
        evaluator.run ();
    }
    
    private void readExercise (InputStream is) throws IOException {
        
        File file = new File (m_tempdir, "exercise.zip");
        System.out.println ("EvalClient.readExercise ()");
        SocketFileUtil.readFile (is, file.getPath ());
    }
    
    private void readSubmission (InputStream is) throws IOException {
        
        File file = new File (m_tempdir, "submission.zip");
        System.out.println ("EvalClient.readSubmission ()");
        SocketFileUtil.readFile (is, file.getPath ());
    }
    
    private void unzipTests() throws IOException {
        File zip_ex = new File (m_tempdir, "exercise.zip");
        no.ntnu.stud.frodesol.jexserver.util.Util.unzipFile (
                zip_ex.getPath (), m_tempdir);
    }
    
    private void unzipSubmission () throws IOException {
        File zip_sub = new File (m_tempdir, "submission.zip");
        File submission_dir = new File (m_tempdir,
                Exercise.getSubmissionDirName ());
        no.ntnu.stud.frodesol.jexserver.util.Util.unzipFile (
                zip_sub.getPath (), submission_dir.getPath ());
    }
    
    private void writeResults (OutputStream os) throws IOException {
        
        System.out.println ("EvalClient.writeResults ()");
        File file = new File (m_tempdir, "report.xml");
        SocketFileUtil.writeFile (os, file.getPath ());
        os.flush ();
    }
    
    public void writeErrorReport (String errormsg)
            throws IOException {
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder docbuilder = factory.newDocumentBuilder ();
            Document doc = docbuilder.newDocument ();
            Element root = doc.createElement ("report");
            doc.appendChild (root);
            
            Element errornode = doc.createElement ("error");
            root.appendChild (errornode);
            errornode.setTextContent (errormsg);
            
            Element testsnode = doc.createElement ("testresults");
            root.appendChild (testsnode);
            
            Node compnode = doc.createElement ("compiler");
            root.appendChild (compnode);
            
            TransformerFactory tfactory = TransformerFactory.newInstance (); 
            Transformer transformer = tfactory.newTransformer (); 
        
            Source src = new DOMSource (doc);
            
            FileOutputStream fos = null;
            try {
                File file = new File (m_tempdir, "report.xml");
                fos = new FileOutputStream (file.getPath());
                Result dest = new StreamResult (fos); 
                transformer.transform (src, dest); 
            }
            finally {
                fos.close ();                
            }
        }
        catch (Exception e) {
            System.out.println (e.getMessage ());
            throw new IOException ("Could not write result: " + e.getMessage ());
        }
        
        System.out.println("Done...");
    }

}
