package no.ntnu.stud.frodesol.jexserver.bin;

import java.io.File;

import no.ntnu.stud.frodesol.jexserver.eval.*;
import no.ntnu.stud.frodesol.jexserver.util.Util;

public class JexTester {
	
	public static void main (String[] args) {
		
	    System.out.println ("");
		System.out.println ("== Jexercise Server - Test Runner           ==");
		System.out.println ("== by Frode Solheim <frodesol@stud.ntnu.no> ==");
		System.out.println ("");
		
		if (args.length < 1) {
			System.err.println ("Missing exercise directory argument");
			System.exit (1);
		}
		
        String dirpath = args[0];
        int start_from = -1;
        int last_test = -1;
        
		if (args.length >= 2) {
			start_from = Integer.parseInt (args[1]);
		}
        if (args.length >= 3) {
            last_test = Integer.parseInt (args[2]);
        }
		
        System.setProperty ("jex.exercisepath", dirpath);
        System.setProperty ("jex.resourcepath", new File (dirpath,
                "resources/").getPath ());
        
        JexTester testrunner = new JexTester (dirpath);
		boolean success = testrunner.run (start_from, last_test);
		if (success) {
			System.out.println ("[DONE]");
		}
		else {
		    System.out.println ("[FAIL]");
		}
	}
	
	private String m_exdir;
	private Exercise m_exercise;
	
	public JexTester (String exdir) {
		m_exdir = exdir;
	}
	
	public boolean run (int start_from, int last_test) {
		
		try {			
			loadExercise ();
			cleanExercise ();
			
			if (start_from == -1) {
				start_from = 0;
			}
			
			// testJavaRequirements ();
			runTests (start_from, last_test);
		}
		catch (Exception e) {
			e.printStackTrace ();
			return false;
		}
		return true;
	}
	
	private void loadExercise () throws ExerciseException {
		
		System.out.println ("JexTester: Loading exercise...");
		m_exercise = new Exercise (m_exdir);
		System.out.println ("JexTester: Loading exercise completed");
	}
	
	private void cleanExercise () throws java.io.IOException {
        
	    // Create or clean results directory
        File fileobj;
        fileobj = new File (m_exercise.getResultsDirectory ());
        if (fileobj.exists ()) {
            Util.deleteFolder (fileobj.getPath ());
        }
        fileobj.mkdir ();
	}
	
	private void runTests (int start_from, int last_test)
	        throws ExerciseException {
		
		System.out.println ("Set class loader");
		System.out.println ("JexTester: runTests");
		
		System.out.println ("get all test nodes");
		TestNode[] nodes = m_exercise.getTree ().getAllTestNodes ();
		System.out.println ("loop through all test nodes");
		System.out.println (nodes);
		System.out.println (nodes.length);
		System.out.println (start_from);
		int points = 0;
		int lastindex = nodes.length - 1;
		if (last_test > -1) {
		    lastindex = last_test;
		}
		for (int i = start_from; i <= lastindex; i++) {
			points += runTest (nodes[i], i);
			m_exercise.writeTestResult (nodes[i], i);
            m_exercise.confirmNodeResult (i);
		}
		System.out.println ("POINTS " + points);
	}
	
	private int runTest (TestNode node, int testnumber) {
		
		String line = "[TEST " + testnumber + "] " + node.getName ();
		if (line.length () > 78) {
			line = line.substring (0, 75) + "...";
		}
		System.out.println (line);
		
		RequirementTester tester = RequirementTester.createTesterForNode (
				m_exercise.getTree (), node);
		try {
			tester.testRequirement ();
			System.out.println ("[RESULT " + testnumber + "] PASS");
			node.setTestResult (true);
			return node.getMaxPoints ();
			
		}
		catch (ExerciseException e) {
			System.out.println ("Exception: " + e.getMessage ());
			System.out.println ("[RESULT " + testnumber + "] FAIL");
			node.setTestResult (false);
		}
			
		return 0;
	}
}
