package no.ntnu.stud.frodesol.jexserver.core;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class Exercise {
    
    public static Exercise get (String primary_key) throws Exception {
        
        Exercise exercise = new Exercise (primary_key);
        exercise.load ();
        return exercise;
    }
    
    public static Exercise findByNameAndCourse (String name, String courseid)
            throws Exception {
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("SELECT " +
                "uuid FROM exercise WHERE name = ? AND course = ?");
        stat.setString (1, name);
        stat.setString (2, courseid);
        ResultSet rs = stat.executeQuery ();
        rs.next ();
        
        try {
            String uuid = rs.getString (1);
            return Exercise.get (uuid);
        }
        catch (Exception e) {
            throw new Exception("Could not find exercise " + name + " for " +
                    "course " + courseid);
        }
        finally {
            rs.close ();
            stat.close ();
        }
        
    }
    
    public static Exercise create (String courseid, String name) 
            throws Exception {
        
        String uuid = java.util.UUID.randomUUID ().toString ();
        int version = 0;
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("INSERT " +
                "INTO exercise (course, name, uuid, version) " +
                "VALUES (?, ?, ?, ?)");
        stat.setString (1, courseid);
        stat.setString (2, name);
        stat.setString (3, uuid);
        stat.setInt (4, version);
        
        stat.execute ();
        stat.close ();
        
        return get (uuid);
    }
    
    private String m_uuid;
    
    private boolean m_loaded = false;
    private int m_version;
    private String m_name;
    
    
    public Exercise (String primary_key) {
        m_uuid = primary_key;
    }
    
    public String getId () {
        return m_uuid;
    }
    
    public int getVersion () throws Exception {
        load ();
        return m_version;
    }
    
    public String getName () throws Exception {
        load ();
        return m_name;
    }
    
    public String getZipFilePath () throws Exception {
        
        load ();
        File f;
        f = new File (JexEnv.getStorage (), "exercise");
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + "-" + m_version + ".zip");
        return f.getPath ();
    }
    
    public String getNewZipFilePath () throws Exception {
        
        load ();
        File f;
        f = new File (JexEnv.getStorage (), "exercise");
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + "-" + (m_version+1) + ".zip");
        return f.getPath ();
    }
    
    public String getDistZipFilePath () throws Exception {
        
        load ();
        File f;
        f = new File (JexEnv.getStorage (), "exercise");
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + "-dist.zip");
        return f.getPath ();
    }
    
    public void updateWithNewZip () throws Exception {
        
        String zippath = getNewZipFilePath ();
        File destfile = new File (zippath.substring (0, 
                zippath.length () - 1 - 3) + ".jex");
        //System.out.println ("Unzip file " + zippath);
        //System.out.println (" - into " + destdir);
        ZipFile zipfile = new ZipFile (zippath);
        Enumeration entries = zipfile.entries ();
        while (entries.hasMoreElements()) {
            
            ZipEntry entry = (ZipEntry) entries.nextElement ();
            if (entry.getName ().compareTo ("exercise.jex") == 0) {
                FileOutputStream fos = new FileOutputStream (destfile.getPath ());
                InputStream is = zipfile.getInputStream (entry);
                byte[] buffer = new byte[16384];
                int read = is.read (buffer);
                while (read > 0) {
                    fos.write (buffer, 0, read);
                    read = is.read (buffer);
                }
                is.close ();
                fos.close ();
                break;
            }
        }
        zipfile.close ();
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("UPDATE " +
                "exercise set version = ? WHERE uuid = ?");
        stat.setInt (1, m_version + 1);
        stat.setString (2, m_uuid);
        stat.execute ();
        stat.getUpdateCount ();
        stat.close ();
        
        m_version = m_version + 1;
    }
    
    public String getJexFilePath () throws Exception {
        
        load ();
        return getJexFilePath (m_version);
    }
    
    public String getJexFilePath (int version) {
        
        File f;
        f = new File (JexEnv.getStorage (), "exercise");
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + "-" + version + ".jex");
        return f.getPath ();
    }
    
    public List<Submission> getSubmissionsByStudent (Student student)
            throws Exception {
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("SELECT " +
                "id FROM submission WHERE exercise = ? AND student = ? " +
                "AND autosubmitted = 0 ORDER BY receivetime DESC");
        stat.setString (1, m_uuid);
        stat.setString (2, student.getUserName ());
        ResultSet rs = stat.executeQuery ();
        
        ArrayList<Submission> list = new ArrayList<Submission> ();
        while (rs.next ()) {
            list.add (new Submission (rs.getInt (1)));
        }
        rs.close ();
        database.close ();
        
        return list;
    } 
    
    public void load () throws Exception {
        
        if (m_loaded) return;
        Database database = Database.getInstance ();
        
        PreparedStatement stat = database.prepareStatement ("SELECT version, " +
                "name FROM exercise WHERE uuid = ?");
        stat.setString (1, m_uuid);
        ResultSet rs = stat.executeQuery ();
        rs.next ();
        m_version = rs.getInt (1);
        m_name = rs.getString (2);
        rs.close ();
        database.close ();
        m_loaded = true;
    }
    
}