package no.ntnu.stud.frodesol.jexserver.core;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import no.ntnu.stud.frodesol.jexserver.eval.*;

public class Report {
    
    private Submission m_submission;
    private TestTree m_testtree;
    private String m_errormsg = null;
    
    public Report (Submission submission) throws Exception {
        
        m_submission = submission;
        
        Exercise exercise = m_submission.getExercise ();
        TestTreeLoader loader = new TestTreeLoader ();
        String jexfilepath = exercise.getJexFilePath (
                m_submission.getExerciseVersion ());
        System.out.println ("Loading report from " + jexfilepath);
        m_testtree = loader.loadExcercise (jexfilepath);
        
        String reportfilepath = m_submission.getReportFilePath ();
        loadTestReport (reportfilepath);
        
    }
    
    public int calculateScore () {
        
        return m_testtree.calculateResult ()[0];
    }
    
    public TestNode[] getTestNodes () {
        
        return m_testtree.getAllTestNodes ();
    }
    
    /**
     * 
     * @return error message, or null if no error.
     */
    public String getErrorMessage() {
        return m_errormsg;
    }
    
    public TestNode getTestRootNode () {
        
        return m_testtree.getRootNode ();
    }
    
    public TestTree getTestTree () {
        
        return m_testtree;
    }
    
    private void loadTestReport (String reportfilepath) throws Exception {
        
        Document document = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
        factory.setNamespaceAware (true);
        DocumentBuilder builder = factory.newDocumentBuilder ();
        document = builder.parse (reportfilepath);
        
        TestNode[] testnodes = getTestNodes ();
        
        Element element = document.getDocumentElement ();
        //addRequirementsNodes (rootnode, element);
        
        System.out.println ("Reading scores");
        
        Element testresults = null;
        NodeList nlist = element.getChildNodes ();
        for (int i = 0; i < nlist.getLength (); i++) {
            org.w3c.dom.Node n = nlist.item (i);
            if (n.getNodeName ().equals ("error")) {
                m_errormsg = n.getTextContent();
            }
            
            if (n.getNodeName ().equals ("testresults") == false) continue;
            testresults = (Element) n;
            break;
        }
        
        int k = 0;
        nlist = testresults.getChildNodes ();
        for (int i = 0; i < nlist.getLength (); i++) {
            
            org.w3c.dom.Node n = nlist.item (i);
            if (n.getNodeName ().equals ("test") == false) continue;
            
            loadTestNode ((Element) n, testnodes, 0);
            break;
        }
    }
    
    private int loadTestNode (Element element, TestNode[] testnodes, 
            int testindex) {

        TestNode testnode = testnodes[testindex];
        
        if (testnode.getMaxPoints () == 0) {
            testnode.setTestResult (true);
        }
        else {
            try {
                testnode.setTestResult (Integer.parseInt (
                        element.getAttribute ("achieved")) > 0);
            }
            catch (Exception e) {
                
            }
        }
        
        NodeList nlist = element.getChildNodes ();
        for (int i = 0; i < nlist.getLength (); i++) {
            org.w3c.dom.Node n = nlist.item (i);
            if (n.getNodeName ().equals ("problems") == false) continue;
            NodeList n2list = ((Element) n).getChildNodes ();
            for (int i2 = 0; i2 < n2list.getLength (); i2++) {
                org.w3c.dom.Node n2 = n2list.item (i2);
                if (n2.getNodeName ().equals ("problem") == false) continue;
                
                String message = null;
                String detail = null;
                NodeList n3list = ((Element) n2).getChildNodes ();
                for (int i3 = 0; i3 < n3list.getLength (); i3++) {
                    org.w3c.dom.Node n3 = n3list.item (i3);
                    if (n3.getNodeName ().equals ("message")) {
                        message = ((Element) n3).getTextContent (); 
                    }
                    else if (n3.getNodeName ().equals ("detail")) {
                        detail = ((Element) n3).getTextContent (); 
                    }
                }
                testnode.addProblem (message, detail);
            }
        }
        
        nlist = element.getChildNodes ();
        for (int i = 0; i < nlist.getLength (); i++) {
            org.w3c.dom.Node n = nlist.item (i);
            if (n.getNodeName ().equals ("test") == false) continue;
            
            testindex = loadTestNode ((Element) n, testnodes, testindex + 1);
        }
        
        return testindex;
    }
}
