package no.ntnu.stud.frodesol.jexserver.core;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.text.DateFormat;
import java.util.*;

public class Student {
    
    public static Student findByCredentials (String username, String password,
                boolean allowcreate)
            throws Exception {
        
        if (password.length() == 0) {
            throw new Exception ("Password was empty");
        }
        
        try {
            Database database = Database.getInstance ();
            PreparedStatement stat = database.prepareStatement ("SELECT " +
            		"username, isadmin FROM student WHERE username = ? " +
            		"AND password = ?");
            stat.setString (1, username);
            stat.setString (2, password);
            ResultSet rs = stat.executeQuery ();
            
            if (rs.next () == false) {
                throw new Exception ("Could not find student");
            }
            boolean isadmin = rs.getBoolean (2);
            rs.close ();
            //database.close ();
            
            return new Student (username, isadmin);
        }
        catch (Exception e) {
            if (allowcreate) {
                return create (username, password);
            }
            throw e;
        }
    }
    
    public static Student findByCredentials (String username,
            boolean allowcreate)
        throws Exception {
    
        try {
            Database database = Database.getInstance ();
            PreparedStatement stat = database.prepareStatement ("SELECT " +
                    "username, isadmin FROM student WHERE username = ?");
            stat.setString (1, username);
            ResultSet rs = stat.executeQuery ();
            
            if (rs.next () == false) {
                throw new Exception ("Could not find student");
            }
            boolean isadmin = rs.getBoolean (2);
            rs.close ();
            //database.close ();
            
            return new Student (username, isadmin);
        }
        catch (Exception e) {
            if (allowcreate) {
                return create (username, "");
            }
            throw e;
        }
    }
    
    public static Student create (String username, String name,
                String data)
            throws Exception {
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("INSERT " +
                "INTO student (username, password, isadmin, name, data) " +
                "VALUES (?, ?, ?, ?, ?)");
        stat.setString (1, username);
        stat.setString (2, "");
        stat.setBoolean (3, false);
        stat.setString (4, name);
        stat.setString (5, data);
        stat.execute ();
        stat.close ();
        return findByCredentials (username, false);
    }
    
    public static Student create (String username, String password)
            throws Exception {
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("INSERT " +
                "INTO student (username, password, isadmin) " +
                "VALUES (?, ?, ?)");
        stat.setString (1, username);
        stat.setString (2, password);
        stat.setBoolean (3, false);
        stat.execute ();
        stat.close ();
        return findByCredentials (username, password, false);
        }
            
    private String m_username;
    private boolean m_isadmin = false;
    
    
    public Student (String username, boolean isadmin) {
        
        m_username = username;
        m_isadmin = isadmin;
    }
    
    public String getUserName () {
        
        return m_username;
    }
    
    public boolean isAdmin () {
        
        return m_isadmin;
    }
    
    public void attendCourse (String courseid) throws Exception {
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("SELECT " +
                "course FROM attends WHERE student = ? AND course = ?");
        stat.setString (1, m_username);
        stat.setString (2, courseid);
        ResultSet rs = stat.executeQuery ();
        
        if (rs.next () == false) {
            PreparedStatement stat2 = database.prepareStatement ("INSERT " +
            "INTO attends (student, course) VALUES (?, ?)");
            stat2.setString (1, m_username);
            stat2.setString (2, courseid);
            stat2.execute ();
            stat2.close ();
        }
        stat.close ();
    }
    
    public List<Course> getCourses() throws Exception {
        
        Database database = Database.getInstance();
        PreparedStatement stat = database.prepareStatement("SELECT " +
                "course FROM attends WHERE student = ?");
        stat.setString(1, m_username);
        ResultSet rs = stat.executeQuery();
        
        ArrayList<Course> list = new ArrayList<Course>();
        while (rs.next()) {
            list.add(new Course(rs.getString(1)));
        }
        rs.close();
        database.close();
        
        return list;
    }
    
    public Submission getBestSubmissionForExercise(Exercise exercise)
            throws Exception {
        
        Database database = Database.getInstance();
        PreparedStatement stat = database.prepareStatement("SELECT id " +
                "FROM submission WHERE exercise = ? AND student = ? AND " +
                "autosubmitted = 0 " +
                "ORDER BY score DESC, receivetime ASC");
        stat.setString(1, exercise.getId());
        stat.setString(2, getUserName());
        ResultSet rs = stat.executeQuery();
        if (rs.next () == false) {
            stat.close();
            rs.close ();
            database.close ();
            return null;
        }
        int submissionid = rs.getInt (1);
        return Submission.get(submissionid);
    }
    
}
