package no.ntnu.stud.frodesol.jexserver.core;

import java.sql.*;
import java.io.File;
import java.text.DateFormat;
import no.ntnu.stud.frodesol.jexserver.eval.*;

public class Submission {
    
    public static Submission get (int primary_key) throws Exception {
        
        Submission submission = new Submission (primary_key);
        submission.load ();
        return submission;
    }
    
    private int m_primary_key;
    private boolean m_loaded = false;
    
    private String m_uuid;
    private Exercise m_exercise = null;
    private String m_exerciseuuid;
    private int m_exversion;
    private java.util.Date m_receivetime;
    private String m_studentusername;
    private boolean m_graded = false;
    private int m_score = 0;
    private boolean m_autosubmitted = false;
    private int m_priority = 0;
    
    public Submission (int primary_key) {
        
        m_primary_key = primary_key;
    }
    
    public Submission (Exercise exercise, Student student) throws Exception {
        
        m_primary_key = 0;
        m_uuid = java.util.UUID.randomUUID ().toString ();
        m_exerciseuuid = exercise.getId ();
        m_exversion = exercise.getVersion ();
        m_studentusername = student.getUserName ();
        // FIXME TIME
        m_loaded = true;
    }
    
    public void setAutosubmitted () {
        m_autosubmitted = true;
        m_priority = -10;
    }
    
    public int getId () {
        return m_primary_key;
    }
    
    public String getUUID () throws Exception {
        load ();
        return m_uuid;
    }
    
    public boolean isGraded () throws Exception {
        load ();
        return m_graded;
    }
    
    public int getScore () throws Exception {
        load ();
        return m_score;
    }
    
    public Exercise getExercise () throws Exception {
        load ();
        if (m_exercise == null) {
            m_exercise = Exercise.get (m_exerciseuuid);
        }
        return m_exercise;
    }
    
    public String formatReceiveTime () throws Exception  {
        load ();
        return m_receivetime.toLocaleString ();
    }
    
    public String getStudentUserName () throws Exception  {
        load ();
        return m_studentusername;
    }
    
    public int getExerciseVersion () throws Exception  {
        load ();
        return m_exversion;
    }
    
    public String getZipFilePath () throws Exception  {
        load ();
        File f;
        f = new File (JexEnv.getStorage (), "submission");
        f = new File (f.getPath (), m_uuid.substring (0, 2));
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + ".zip");
        return f.getPath ();
    }
    
    public String getReportFilePath () throws Exception  {
        load ();
        File f;
        f = new File (JexEnv.getStorage (), "submission");
        f = new File (f.getPath (), m_uuid.substring (0, 2));
        if (f.exists () == false) {
            f.mkdirs ();
        }
        f = new File (f.getPath (), m_uuid + ".xml");
        return f.getPath ();
    }
    
    public void setExerciseVersion (int version) throws Exception  {
        
        m_exversion = version;
    }
    
    public Report createReport () throws Exception {
        load ();
        //String path = getReportFilePath ();
        Report report = new Report (this);
        return report;
    }
    
    public void writeResultsToDatabase () throws Exception {
        
        Report report = createReport ();
        int score = report.calculateScore ();
        System.out.println ("Submission.saveResultsToDatabase, " +
        		"score = " + score);
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("UPDATE " +
                "submission set exversion = ?, " +
                "gradeendtime = UTC_TIMESTAMP(), " +
                "score = ? WHERE id = ?");
        stat.setInt (1, m_exversion);
        stat.setInt (2, score);
        stat.setInt (3, m_primary_key);
        stat.execute ();
        stat.getUpdateCount ();
        stat.close ();
            
        stat = database.prepareStatement ("DELETE FROM scoring WHERE " +
        		"submission = ?");
        stat.setInt (1, m_primary_key);
        stat.execute ();
        stat.close ();
        
        TestNode[] nodes = report.getTestNodes ();
        for (int i = 0; i < nodes.length; i++) {
            if (nodes[i].getMaxPoints () == 0) continue;
            stat = database.prepareStatement ("INSERT " +
                    "INTO scoring (submission, test, points) VALUES (?, ?, " +
                    "?)");
            stat.setInt (1, m_primary_key);
            stat.setInt (2, i);
            stat.setInt (3, nodes[i].getAchievedPoints ());
            stat.execute ();
            stat.close ();
        }
    }
    
    public void load () throws Exception {
        
        if (m_loaded) return;
        
        Database database = Database.getInstance ();
        PreparedStatement stat = database.prepareStatement ("SELECT uuid, " +
        		"exercise, exversion, receivetime, student, " +
        		"gradestarttime, gradeendtime, score " +
        		"FROM submission WHERE id = ?");
        stat.setInt (1, m_primary_key);
        ResultSet rs = stat.executeQuery ();
        rs.next ();
        m_uuid = rs.getString (1);
        m_exerciseuuid = rs.getString (2);
        m_exversion = rs.getInt (3);
        
        m_studentusername = rs.getString (5);
        
        String receivetime = rs.getString (4);
        DateFormat df = new SqliteDateFormat ();
        m_receivetime = df.parse (receivetime + " UTC");
        //m_receivetime = df.parse (receivetime);
        
        if (rs.getString (8) == null) {
            m_score = 0;
            m_graded = false;
        }
        else {
            m_score = rs.getInt (8);
            m_graded = true;
        }
        rs.close ();
        database.close ();
        m_loaded = true;
    }
    
    public void save () throws Exception {
        
        Database database = Database.getInstance ();
        if (m_primary_key == 0) {
            
            PreparedStatement stat = database.prepareStatement ("INSERT " +
            		"INTO submission (uuid, exercise, student, receivetime, " +
            		"exversion, priority, autosubmitted) VALUES (?, ?, ?, UTC_TIMESTAMP(), ?, ?, ?)");
            stat.setString (1, m_uuid);
            stat.setString (2, m_exerciseuuid);
            stat.setString (3, m_studentusername);
            stat.setInt (4, m_exversion);
            stat.setInt (5, m_priority);
            stat.setBoolean (6, m_autosubmitted);
            
            stat.execute ();
            ResultSet rs = stat.getGeneratedKeys ();
            rs.next ();
            m_primary_key = rs.getInt (1);
            rs.close ();
            stat.close ();
            
            //saveScores ();
        }
        else {
            // Update is not supportet yet
        }
        database.close ();
    }
    /*
    private void saveScores () throws Exception {
        

    }
    */
}
