package no.ntnu.stud.frodesol.jexserver.dispatch;

import java.io.*;
import java.net.*;
import no.ntnu.stud.frodesol.jexserver.*;
import no.ntnu.stud.frodesol.jexserver.core.*;
import no.ntnu.stud.frodesol.jexserver.util.SocketFileUtil;

public class DispatchThread extends Thread {
    
    static final int BUFFER_SIZE = 65536;
    
    private Socket m_socket;
    private OutputStream m_output;
    private InputStream m_input;
    private Dispatcher m_dispatcher;
    private int m_nextjob = -1;
    
    public DispatchThread (Dispatcher dispatcher, Socket socket)
            throws IOException {
        
        System.out.println ("Creating new DispatchThread");
        
        m_dispatcher = dispatcher;
        m_socket = socket;
        m_input = socket.getInputStream ();
        m_output = socket.getOutputStream ();
    }
    
    public void setNextJob (int nextjob) {
        
        m_nextjob = nextjob;
    }
    
    public int getNextJob () {
        
        int job = m_nextjob;
        m_nextjob = -1;
        return job;
    }
    
    public void run () {
        
        System.out.println ("  -- " + this + " New DispatchThread started ");
        while (true) {
            int job = -1;
            try {
                System.out.println ("  -- " + this + " Calling waitForJob");
                job = m_dispatcher.waitForJob (this);
            }
            catch (Exception e) {
                e.printStackTrace ();
                continue;
            }
            System.out.println ("  -- " + this + " Got job " + job);
            try {
                processJob (job);
            }
            catch (Exception e) {
                e.printStackTrace ();
                break;
            }
        }
        System.out.println ("  -- " + this + " DispatchThread exiting ");
        
        try {
            m_socket.close ();
        }
        catch (Exception e) {}
    }
    
    private void processJob (int job) throws Exception {
        
        try {
            long t1 = System.currentTimeMillis ();
            
            Submission submission = Submission.get (job);
            Exercise exercise = submission.getExercise ();
            
            writeExercise (exercise);
            
            writeSubmission (submission);
            readResults (submission);
            
            submission.setExerciseVersion (exercise.getVersion ());
            submission.writeResultsToDatabase ();
            
            long t2 = System.currentTimeMillis ();
            long t = t2 - t1;
            System.out.println ("Round-trip time: " + t);
        }
        catch (Exception e) {
            throw e;
        }
    }
    
    private void writeExercise (Exercise exercise) throws Exception  {
        
        System.out.println ("  (write exercise)");
        String path = exercise.getZipFilePath ();
        SocketFileUtil.writeFile (m_output, path);
        m_output.flush ();
    }

    private void writeSubmission (Submission submission) throws Exception  {
        
        System.out.println ("  (write submission)");
        String path = submission.getZipFilePath ();
        SocketFileUtil.writeFile (m_output, path);
        m_output.flush ();
    }
    
    private void readResults (Submission submission) throws Exception {
        
        System.out.println ("  (waiting for results)");
        String path = submission.getReportFilePath ();
        SocketFileUtil.readFile (m_input, path);
    }
    
    
}
