package no.ntnu.stud.frodesol.jexserver.dispatch;

import java.io.*;
import java.net.*;
import java.sql.*;

import no.ntnu.stud.frodesol.jexserver.*;
import no.ntnu.stud.frodesol.jexserver.core.*;

public class Dispatcher {
    
    private java.util.LinkedList<DispatchThread> m_waitlist = 
            new java.util.LinkedList<DispatchThread> ();
    
    public void run () throws Exception {
        
        // Force connection to database
        
        Database.getInstance ();
        
        // Create server listening port
        
        int port = JexEnv.getDispatcherPort ();
        ServerSocket listener;
        try {
            listener = new ServerSocket (port);
        }
        catch (IOException e) {
            System.out.println ("Could not open listening socket");
            return;
        }
        
        // Create a separate listening thread to accept new connections
        // and spawn new worker threads
        
        ListenerThread listenerthread = new ListenerThread (this, listener);
        listenerthread.start ();
        
        // High priority queue
        JobQueue highqueue = new JobQueue (0, 100);
        JobQueue lowqueue = new JobQueue (-100, -1);
        
        while (true) {
            
            highqueue.updateFromDatabase ();
            boolean foundsome = false;
            while (highqueue.hasMore ()) {
                foundsome = true;
                assignJob (highqueue.getNextJob ());
            }
            
            lowqueue.updateFromDatabase ();
            while (lowqueue.hasMore ()) {
                highqueue.updateFromDatabase ();
                if (highqueue.hasMore ()) {
                    break;
                }
                foundsome = true;
                assignJob (lowqueue.getNextJob ());
            }
            
            // The following delay causes a slight delays in grading for 
            // a submission during very low load (when the queue gets empty
            // often), but is nicer to the operating system.
            
            if (foundsome == false) {
                try {
                    Thread.sleep (500);
                }
                catch (InterruptedException e) {}
            }
        }
    }
    
    public void assignJob (int job) throws Exception {
        
        DispatchThread thread = null;
        while (thread == null) {
            // FIXME: Wait on the list instead, if it is empty..
            
            boolean empty = true;
            synchronized (m_waitlist) {
                empty = m_waitlist.isEmpty ();
                if (empty == false) {
                    thread = m_waitlist.removeFirst ();
                }
            }
            if (empty) {
                try {
                    Thread.sleep (10);
                }
                catch (InterruptedException e) {}
            }
        }
        
        Database database = Database.getInstance ();
        synchronized (database) {
            PreparedStatement stat = database.prepareStatement ("UPDATE " +
                    "submission SET gradestarttime = UTC_TIMESTAMP() " +
                    "WHERE id = ?");
            stat.setInt (1, job);
            stat.execute ();
            stat.getUpdateCount ();
            stat.close ();
            //database.close ();
        }
        synchronized (thread) {
            thread.setNextJob (job);
            thread.notify ();
        }
        
        System.out.println ("Gave job " + job + " to thread " + thread);
    }
    
    public int waitForJob (DispatchThread thread) throws Exception {
        
        System.out.println (".");
        synchronized (thread) {
            System.out.println ("..");
            synchronized (m_waitlist) {
                System.out.println ("...");
                m_waitlist.add (thread);
            }
            System.out.println ("Thread " + thread + " waiting for job");
            thread.wait ();
            return thread.getNextJob ();
        }
    }

}
