package no.ntnu.stud.frodesol.jexserver.dispatch;

import java.sql.*;

import no.ntnu.stud.frodesol.jexserver.core.Database;

public class JobQueue {
    
    private int m_max_id;
    private int m_maxpri;
    private int m_minpri;
    private java.util.LinkedList<Integer> m_joblist;
    
    public JobQueue (int minpri, int maxpri) {
        
        m_minpri = minpri;
        m_maxpri = maxpri;
        m_max_id = -1;
        m_joblist = new java.util.LinkedList<Integer> ();
    }
    
    public synchronized int getNextJob () {
        
        return m_joblist.removeFirst ();
    }

    public synchronized boolean hasMore () {
        
        return !m_joblist.isEmpty ();
    }
    
    public synchronized boolean isEmpty () {
        
        return m_joblist.isEmpty ();
    }
    
    public synchronized int size () {
        
        return m_joblist.size ();
    }
    
    private synchronized void updateFromResultSet (ResultSet rs) 
            throws Exception {
        
        // m_max_id is used for efficiency reasons. Since we order results
        // from database, we can skip looking up jobs already in the job
        // queue by adding id > m_max_id to the query, and we do not have
        // to check the results against the local queue
        
        while (rs.next ()) {
            int submission = rs.getInt (1);
            System.out.println (" ------ added job " + submission + 
                    " to queue");
            m_max_id = submission;
            m_joblist.add (m_max_id);
        }
        rs.close ();
    }
    
    public void updateFromDatabase () throws Exception {
        
        // If queue is empty, reset m_max_id, just in case there are (for
        // for some reason) ungraded jobs not in the local queue with id
        // below m_max_id. This could happen if a DispatchThread/EvalClient
        // died unexpectedly.
        
        if (isEmpty ()) {
            m_max_id = -1;
        }
                
        Database database = Database.getInstance ();
        
        PreparedStatement stat = database.prepareStatement ("SELECT id FROM " +
                "submission WHERE score IS NULL AND " +
                "(gradestarttime IS NULL OR now() > " +
                "TIMESTAMPADD (SECOND, 300, gradestarttime)) AND " +
                "id > ? AND priority >= ? AND priority <= ? ORDER BY id");
        stat.setInt (1, m_max_id);
        stat.setInt (2, m_minpri);
        stat.setInt (3, m_maxpri);
        
        ResultSet rs = stat.executeQuery ();
        updateFromResultSet (rs);
        stat.close ();
        //database.close ();
    }
}
