package no.ntnu.stud.frodesol.jexserver.eval;

import java.io.*;

import no.ntnu.stud.frodesol.jexserver.core.JexEnv;

public class Exercise {
	
	private String m_exdir = "";
	private TestTree m_extree;
	
	public Exercise (String exdir) throws ExerciseException {
		
		m_exdir = exdir;
		TestTreeLoader loader = new TestTreeLoader ();
		m_extree = loader.loadFromDir (m_exdir);
		
	}
	
	public String getPath () {
		return m_exdir;
	}
	
	public TestTree getTree () {
		return m_extree;
	}

    public String createTempDir () throws IOException {
        File fname = new File (getPath (), "temp");
        fname.mkdir ();
        return fname.getPath ();
    }
	
	public String createPolicyFile () throws IOException {
	    
        File fname = new File (getPath (), "policy");
        FileWriter fw = new FileWriter (fname.getPath ());
        String p;
        
        p = JexEnv.getHome ().replace ('\\', '/') + "/bin/";
        fw.write ("grant codeBase \"file:" + p + "\" {\n");
        fw.write ("  permission java.security.AllPermission;\n");
        fw.write ("};\n");
        fw.write ("\n");
        
        p = JexEnv.getHome ().replace ('\\', '/') + "/junit.jar";
        fw.write ("grant codeBase \"file:" + p + "\" {\n");
        fw.write ("  permission java.security.AllPermission;\n");
        fw.write ("};\n");
        fw.write ("\n");

        p = JexEnv.getHome ().replace ('\\', '/') + "/jexserver.jar";
        fw.write ("grant codeBase \"file:" + p + "\" {\n");
        fw.write ("  permission java.security.AllPermission;\n");
        fw.write ("};\n");
        fw.write ("\n");
        
        p = getTestBinPath ().replace ('\\', '/') + "/";
        fw.write ("grant codeBase \"file:" + p + "\" {\n");
        fw.write ("  permission java.security.AllPermission;\n");
        fw.write ("};\n");
        fw.write ("\n");
        
        p = getSubmissionBinPath ().replace ('\\', '/') + "/";
        fw.write ("grant codeBase \"file:" + p + "\" {\n");
        
        String d = getResourcePath ();
        d = d.replace ('\\', '/') + "/";
        fw.write ("  permission java.io.FilePermission \"" + d + "-\", \"read\";\n");
        d = new File (getPath (), "temp").getPath ();
        d = d.replace ('\\', '/') + "/";
        fw.write ("  permission java.io.FilePermission \"" + d + "-\", \"write\";\n");

        // These are required by AWT to initialize
        fw.write ("  permission java.lang.RuntimePermission \"shutdownHooks\";\n");
        fw.write ("  permission java.lang.RuntimePermission \"modifyThread\";\n");
        fw.write ("  permission java.util.PropertyPermission \"user.dir\", \"read\";\n");
        fw.write ("  permission java.util.PropertyPermission \"user.home\", \"read\";\n");
        d = System.getProperty ("user.home");
        fw.write ("  permission java.io.FilePermission \"" + d + "\", \"read\";\n");
        
        
        fw.write ("};\n");
        fw.close ();
        return fname.getPath ();
	}
	
	public String getResultsDirectory () {
		File fname = new File (getPath (), "results");
		return fname.toString ();
	}
	
    public String getTestStartFilePath () {
        File fname = new File (getResultsDirectory (), "start");
        return fname.getPath ();
    }	
	
	public String getTestOkFilePath (int num) {
	    File fname = new File (getResultsDirectory (), "result" + num + ".ok");
	    return fname.getPath ();
	}
	
   public String getTestResultFilePath (int num) {
        File fname = new File (getResultsDirectory (), "result" + num + ".xml");
        return fname.getPath ();
    }
   
	public void confirmNodeResult (int num) throws ExerciseException {
	    
	    try {
    	    
    	    FileOutputStream fos = new FileOutputStream (getTestOkFilePath (num));
    	    fos.write ('k');
    	    fos.close ();
	    }
	    catch (java.io.IOException e) {
	        throw new ExerciseException ();
	    }
	}
    
    public void readTestResult (int num) throws ExerciseException {
        TestNode[] nodes = m_extree.getAllTestNodes ();
        TestNode node = nodes[num];
        readTestResult (node, num);
    }	
	
    public void readTestResult (TestNode node, int num) throws ExerciseException {
        node.readResult (getTestResultFilePath (num));
    }

	
	public void writeTestResult (TestNode node, int num) throws ExerciseException {
		node.writeResult (getTestResultFilePath (num));
	}
	
	
	public static String getSubmissionDirName () {
	    return "submission";
	}
	
	public String getSubmissionPath () {
		File fname = new File (getPath (), getSubmissionDirName ());
		return fname.toString ();
	}
	
	public String getSubmissionBinPath () {
		File fname = new File (getPath (), "bin");
		return fname.toString ();
	}

    public String getResourcePath () {
        File fname = new File (getPath (), "resources");
        return fname.toString ();
    }

	
	public String getTestBinPath () {
		File fname = new File (getPath (), "testbin");
		return fname.toString ();
	}
	
	public String getTestsPath () {
		File fname = new File (getPath (), "tests");
		return fname.toString ();
	}
	
	public String getJexFilePath () {
		File fname = new File (getPath (), "exercise.jex");
		return fname.toString ();
	}
	
	public void writeReport () throws IOException {
	    m_extree.writeReport (getReportFilePath (),
	            new File (getPath (), "compiler.xml").getPath ());
	}
	
    public String getReportFilePath () {
        File fname = new File (getPath (), "report.xml");
        return fname.toString ();
    }
	
	public String getJarsPath () {
		File fname = new File (getPath (), "jars");
		return fname.toString ();
	}
	
	public String getJarPath (String jarname) {
		File fname = new File (getJarsPath (), jarname);
		return fname.toString ();
	}
}
