package no.ntnu.stud.frodesol.jexserver.eval;

import java.io.*;

import no.ntnu.stud.frodesol.jexserver.core.JexEnv;
import no.ntnu.stud.frodesol.jexserver.eval.*;

import java.io.File;
import java.util.ArrayList;

public class ExerciseCompiler {
	
	private Exercise m_exercise;
	
	public ExerciseCompiler (Exercise exercise) {
		
		m_exercise = exercise;
	}
	
	public String getCompilerLogFilePath () {
	    
        File file = new File (m_exercise.getPath (), "compiler.xml");
	    return file.getPath ();
	}
	
	public void compileSubmission () throws IOException {
		
		System.out.println (" -- trace -- ExerciseCompiler.compileSubmission");
		
		File tmpfile = File.createTempFile ("jexc", null);
		System.out.println (tmpfile.getPath ());
		
		ArrayList sourcelist = new ArrayList ();
		getSourceFiles (m_exercise.getSubmissionPath (), sourcelist);
		writeSourceFileList (sourcelist, tmpfile.getPath ());
		
		String[] args = {
                "-log",
                getCompilerLogFilePath (),
		        "-proceedOnError",
				"-source",
				"6.0",
				"-noExit",
				"-verbose",
				"-d",
				m_exercise.getSubmissionBinPath (),
				"-sourcepath",
				m_exercise.getSubmissionPath (),
				"@" + tmpfile.getPath (),
			};
		doCompile (args);
	}
	
	public void compileTests () throws IOException {
		
		System.out.println (" -- trace -- ExerciseCompiler.compileTests");
		
		File tmpfile = File.createTempFile ("jexc", null);
		System.out.println (tmpfile.getPath ());
		ArrayList sourcelist = new ArrayList ();
		getSourceFiles (m_exercise.getTestsPath (), sourcelist);
		writeSourceFileList (sourcelist, tmpfile.getPath ());
		
		ArrayList cplist = new ArrayList ();
		cplist.add (m_exercise.getJarPath ("junit.jar"));
		cplist.add (m_exercise.getSubmissionBinPath ());
		
		String sep = System.getProperty ("path.separator");
		String cpstring = "";
	    for (int i = 0; i < cplist.size (); i++) {
	    	if (cpstring.length () > 0) {
	    		cpstring = cpstring + sep + (String) cplist.get (i); 
	    	}
	    	else {
	    		cpstring = (String) cplist.get (i); 
	    	}
	    }
		
		String[] args = {
		        "-log",
		        getCompilerLogFilePath (),
				"-proceedOnError",
				"-noExit",
				"-source",
				"6.0",
				"-verbose",
				"-g",
				"-d",
				m_exercise.getTestBinPath (),
				"-classpath",
				cpstring,
				"-sourcepath",
				m_exercise.getTestsPath (),
				"@" + tmpfile.getPath (),
			};
	    
		try {
			org.eclipse.jdt.internal.compiler.batch.Main.main (args);
		}
		catch (Exception e) {
			
		}
	}
	
	private void doCompile (String[] args) throws IOException {
		System.out.println (" -- trace -- doCompile");
		
		org.eclipse.jdt.internal.compiler.batch.Main.main (args);
		System.out.println ("Compilation completed");
	}
	
	private void getSourceFiles (String dirpath, ArrayList sourcelist) throws IOException {
		
		System.out.println (" -- trace -- getSourceFiles " + dirpath);
		
		File f = new File (dirpath);
		String[] dirlist = f.list ();
		for (int i = 0; i < dirlist.length; i++) {
			String name = dirlist[i];
			File f2 = new File (dirpath, name);
			if (f2.isDirectory ()) {
				getSourceFiles (f2.getPath (), sourcelist);
			}
			else if (name.endsWith (".java")) {
				sourcelist.add (f2.getPath ());
			}
		}
	}
	
	private void writeSourceFileList (ArrayList sourcelist, String filepath) 
			throws IOException {
	    //try {
		FileOutputStream fos = new FileOutputStream (filepath);
	    PrintStream ps = new PrintStream (fos);
	    for (int i = 0; i < sourcelist.size (); i++) {
	    	String s = (String) sourcelist.get (i);
	    	ps.println (s);
	    }
	    ps.close ();
	}
	
	private void cleanOutputDirectory () throws IOException {
		
		System.out.println (" -- trace -- cleanOutputDirectory");
		
		String dirpath = m_exercise.getSubmissionBinPath ();
		cleanDirectory (dirpath);
	}
	
	private void cleanDirectory (String dirpath) throws IOException {
		
		System.out.println (" -- trace -- cleanDirectory " + dirpath);
		
		File f = new File (dirpath);
		String[] dirlist = f.list ();
		for (int i = 0; i < dirlist.length; i++) {
			String name = dirlist[i];
			File f2 = new File (dirpath, name);
			if (f2.isDirectory ()) {
				cleanDirectory (f2.getPath ());
			}
			else if (name.endsWith (".class")) {
				boolean result = f2.delete ();
				if (result == false) {
					throw new java.io.IOException (
							"Could not delete class file " + f2.getPath ());
				}
			}
		}
	}
	
	public void compileEverything () throws IOException {
		
		System.out.println (" -- trace -- compileEveryThing");
		
		cleanOutputDirectory ();
		compileSubmission ();
		//compileTests ();
	}
}
