package no.ntnu.stud.frodesol.jexserver.eval;

import org.w3c.dom.Element;
import java.lang.reflect.*;
import junit.framework.*;

public class JunitTester extends RequirementTester {
	
	public JunitTester (TestTree tree,
			TestNode node) {
		
		super (tree, node);
	}
	
	public void  testRequirement () throws ExerciseException {
		
		System.out.println ("JunitTester.testRequirement");
		
		Element javaelement = getJavaElement ();
		System.out.println (javaelement.getNodeName ());

		try {
			String type = javaelement.getAttribute ("xsi:type");
			if (type.equals ("jex.junit:JavaClassTester")) {
				testClass (javaelement);
				return;
			}
			else if (type.equals ("jex.junit:JavaMethodTester")) {
				testMethod (javaelement);
				return;
			}
			
			throw new Error ("Unknown JavaRequirement: " + type);
		}
		catch (NoClassDefFoundError e) {
			throw new TestException (e.getMessage ());
		}
	}
	
	protected void testClass (Element javaelement)
	        throws TestException, ExerciseException {
		
		System.out.println (" -- trace -- testClass");
		//System.exit (1);
		//return false;
		
        Class c;
        try {
            c = getJavaClass (javaelement);
        }
        catch (ClassNotFoundException e) {
            throw new TestException ("Could not find test class: " + 
                    e.getMessage ());
        }
        System.out.println ("Found class " + c.toString ());
        
        runJUnitTest (c, null);
	}
	
    protected void testMethod (Element javaelement)
            throws TestException, ExerciseException {
	
        Class c;
        try {
            c = getJavaClass (javaelement);
        }
        catch (ClassNotFoundException e) {
            throw new TestException ("Could not find test class: " + 
                    e.getMessage ());
        }
        System.out.println ("Found class " + c.toString ());
        
        String methodname = javaelement.getAttribute ("name");
        java.lang.reflect.Method m;
        
        System.out.println (" -- trace -- d0");
        
        try {
            m = c.getMethod (methodname);
        }
        catch (NoSuchMethodException e) {
            throw new TestException ("Could not find method " + methodname);
        }
        System.out.println ("Found method " + methodname);
        
        runJUnitTest (c, m);
    }
        
	protected void testMethod_DELETE (Element javaelement)
			throws TestException, ExerciseException {
		
		Class c;
		try {
			c = getJavaClass (javaelement);
		}
		catch (ClassNotFoundException e) {
			throw new TestException ("Could not find test class: " + 
					e.getMessage ());
		}
		System.out.println ("Found class " + c.toString ());
		
		String methodname = javaelement.getAttribute ("name");
		java.lang.reflect.Method m;
		
		System.out.println (" -- trace -- d0");
		
		try {
			m = c.getMethod (methodname);
		}
		catch (NoSuchMethodException e) {
			throw new TestException ("Could not find method " + methodname);
		}
		System.out.println ("Found method " + methodname);
		
		System.out.println (" -- trace -- d1");
		
		TestCase testcase;
		try {
			testcase = (TestCase) c.newInstance ();
		}
		catch (Exception e) {
			throw new TestException ("Could not instantiate class, " +
			        e.getMessage ());
		}
		
		System.out.println (" -- trace -- d2");
		
		testcase.setName (methodname);
		
		System.out.println (" -- trace -- d3");
		
		System.out.println ("--trace -- about to start junit test");
		System.out.println (testcase);
		
		TestResult result;
		result = testcase.run ();
		
		if (result.wasSuccessful () == false) {
			System.out.println ("--trace -- JUnit test failed");
			
			System.out.println ("-");
			for (java.util.Enumeration e = result.failures ();
					e.hasMoreElements () ;) {
		        TestFailure t = (TestFailure) e.nextElement (); 
				System.out.println (t);
				System.out.println (t.trace ());
				
				getNode ().addProblem (t.toString (), t.trace ());

		    }
			for (java.util.Enumeration e = result.errors ();
					e.hasMoreElements () ;) {
		        TestFailure t = (TestFailure) e.nextElement (); 
				System.out.println (t);
				System.out.println (t.trace ());
				
				getNode ().addProblem (t.toString (), t.trace ());
		    }
			
			System.out.println ("-");
			throw new TestException ("JUnit test failed");
			
		}
		System.out.println ("--trace -- JUnit test succeeded");
		//return true;
	}
	
    private void runJUnitTest (Class c, Method m)
            throws TestException, ExerciseException {
    
        Test test;
        if (m == null) {
            System.out.println (" -- trace -- This is a test class");
            TestSuite ts = new TestSuite (c);
            test = ts;
        }
        else {
            TestCase testcase;
            try {
                testcase = (TestCase) c.newInstance ();
            }
            catch (Exception e) {
                throw new TestException ("Could not instantiate class, " +
                        e.getMessage ());
            }
            catch (Error e) {
                throw new TestException ("Could not instantiate class, " +
                        e.getMessage ());
            }
            
            System.out.println (" -- trace -- Setting test method to " + 
                    m.getName ());
            testcase.setName (m.getName ());
            test = testcase;
        }
        
        System.out.println ("--trace -- about to start junit test");
        
        TestResult result = new TestResult ();
        test.run (result);
        
        if (result.wasSuccessful () == false) {
            System.out.println ("--trace -- JUnit test failed");
            
            System.out.println ("-");
            for (java.util.Enumeration e = result.failures ();
                    e.hasMoreElements () ;) {
                TestFailure t = (TestFailure) e.nextElement (); 
                System.out.println (t);
                System.out.println (t.exceptionMessage ());
                System.out.println (t.trace ());
                
                getNode ().addProblem (t.toString (), t.trace ());
        
            }
            for (java.util.Enumeration e = result.errors ();
                    e.hasMoreElements () ;) {
                TestFailure t = (TestFailure) e.nextElement (); 
                System.out.println (t);
                System.out.println (t.exceptionMessage ());
                System.out.println (t.trace ());
                
                getNode ().addProblem (t.toString (), t.trace ());
            }
            
            System.out.println ("-");
            throw new TestException ("JUnit test failed");
            
        }
        System.out.println ("--trace -- JUnit test succeeded");
    }
}
