package no.ntnu.stud.frodesol.jexserver.eval;

import org.w3c.dom.*;

public abstract class RequirementTester {
	
	public static RequirementTester createTesterForNode (TestTree tree,
			TestNode node) {
		
		System.out.println ("Node type: " + node.getType ());
		
		String type = node.getType ();
		//f (type.endsWith ("JunitTest"))
		if (type.equals ("jex.ex:JUnitTest")) {
			return new JunitTester (tree, node);
		}
		else if (type.equals ("jex.ex:JavaRequirement")) {
			return new JavaRequirementTester (tree, node);
		}
		else if (type.equals ("jex.ex:ExercisePart")) {
			return new NullRequirementTester ();
		}
		else if (type.length () == 0) {
			return new NullRequirementTester ();
		}
		
		throw new Error ("Unknown exercise type: " + type);
	}
	
	protected TestTree m_tree;
	private TestNode m_node;
	
	protected TestNode getNode () {
	    return m_node;
	}
	
	public RequirementTester (TestTree tree,
			TestNode node) {
		
		m_tree = tree;
		m_node = node;
	}
	
	public abstract void testRequirement () throws ExerciseException;
	
	protected Element getJavaElement () throws ExerciseException {
		
		Document document = m_tree.getJexDocument ();
		Element docroot = document.getDocumentElement ();
		String jepath = m_node.getJavaElementPath ();
		// Use substring (1) to remove first slash in path
		return getJavaElement (docroot, jepath.substring (1));
	}
	
	private Element getJavaElement (Element parent, String jepath)
			throws ExerciseException {
		
		String elementinfo = "";
		int atindex = jepath.indexOf ('/', 1);
		if (atindex == -1) {
			elementinfo = jepath.substring (2);
		}
		else {
			elementinfo = jepath.substring (2, atindex);
		}
		
		String[] splitted = elementinfo.split ("\\.");
		String ename = splitted[0];
		int enumber = Integer.parseInt (splitted[1]);
		
		Element element = null;
		
		int occurrence = 0;
		NodeList nlist = parent.getChildNodes ();
		for (int i = 0; i < nlist.getLength (); i++) {
			org.w3c.dom.Node n = nlist.item (i);
			if (n.getNodeName ().equals (ename) == false) continue;
			
			if (occurrence == enumber) {
				element = (Element) n;
				break;
			}
			
			occurrence++;
		}
		
		if (element == null) {
			throw new ExerciseException ("Did not find element " + elementinfo);
		}
		
		if (atindex > -1) {
			return getJavaElement (element, jepath.substring (atindex));
		}
		else {
			return element;
		}
	}
	
	protected Class getJavaClass (Element javaelement)
			throws ClassNotFoundException, ExerciseException {
		
		String classname = getClassQName (javaelement);
		return Class.forName (classname);
	}
	
	protected String getClassQName (Element javaelement)
			throws ExerciseException {
		String packagename = getPackageName (javaelement);
		if (packagename.length () == 0) return getClassName (javaelement);
		else return packagename + "." + getClassName (javaelement);
	}
	
	protected String getClassName (Element javaelement)
			throws ExerciseException {
	
		if (javaelement.getNodeName ().equals ("classes") == false) {
			if (javaelement.getParentNode () != null &&
					javaelement.getParentNode ().getNodeType () == 
					org.w3c.dom.Node.ELEMENT_NODE) {
				return getClassName ((Element) javaelement.getParentNode ());
			}
			else {
				throw new ExerciseException ("Could not find class name");
			}
		}
		
		return javaelement.getAttribute ("name");
	}
	
	protected String getPackageName (Element javaelement) 
			throws ExerciseException {
	
		if (javaelement.getNodeName ().equals ("javaElements") == false) {
			if (javaelement.getParentNode () != null &&
					javaelement.getParentNode ().getNodeType () == 
					org.w3c.dom.Node.ELEMENT_NODE) {
				return getPackageName ((Element) javaelement.getParentNode ());
			}
			else {
				throw new ExerciseException ("Could not find java package");
			}
		}
		
		return javaelement.getAttribute ("name");
	}

}
