/**
 * 
 */
package no.ntnu.stud.frodesol.jexserver.eval;

import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.*; 
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.stream.StreamResult;

import java.io.*;
import java.util.*;

public class TestNode {
	
	private String m_name = "";
	private String m_type = "";
	private String m_javaelementpath = "";
	private ArrayList m_children = new ArrayList ();
	private ArrayList<Problem> m_problems = new ArrayList<Problem> ();
	private boolean m_tested = false;
	private boolean m_ok = false;
	private int m_points = 0;
	
	public TestNode (String name) {
		
		m_name = name;
	}
	
	public TestNode (String name, String type, String javaelementpath) {
		
		m_name = name;
		m_type = type;
		m_javaelementpath = javaelementpath;
	}
	
	public String getName () {	
		return m_name;
	}
	
	public String getType () {
		return m_type;
	}
	
	public String getJavaElementPath () {
		return m_javaelementpath;
	}
	
	public void addChild (TestNode node) {
		m_children.add (node);
	}
	
	public TestNode[] getChildren () {
		return (TestNode[]) m_children.toArray (new TestNode[0]);
	}
	
	public int getMaxRunningTimeMillis () {
	    // 5 seconds
	    return 5 * 1000;
	}
	
	public boolean passed () {
	    return m_ok;
	}
	
	public int getMaxPoints () {
		return m_points;
	}
	
	public void setMaxPoints (int points) {
		m_points = points;
	}
	
	public int getAchievedPoints () {
		
		if (m_tested == false) {
			// FIXME: Exception here?
		}
		
		if (m_ok) return m_points;
		return 0;
	}
	
	public ArrayList<Problem> getProblems () {
	    return m_problems;
	}
	
	public void setTestResult (boolean passed) {
	    m_ok = passed;
	}
	
	public void addProblem (String message, String detail) {
	    Problem problem = new Problem ();
	    problem.message = message;
	    problem.detail = detail;
	    m_problems.add (problem);
	}
	
	public void readResult (String filepath) throws ExerciseException {
		
		//System.out.println (" -- read result --");
        Document document = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
        factory.setNamespaceAware (true);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder ();
            document = builder.parse (filepath);
        } catch (Exception e) {
            throw new ExerciseException ("Failure loading " + filepath + 
                    ": " + e.getMessage ());
        }
        
        Element element = document.getDocumentElement ();
        String result = element.getAttribute ("passed");
        if (result.compareTo ("yes") == 0) {
            m_ok = true;
        }
        else {
            m_ok = false;
        }
        m_tested = true;
        
        NodeList children = element.getChildNodes ();
        for (int i = 0; i < children.getLength (); i++) {
            Node n1 = children.item (i);
            if (n1.getNodeName () == "problems") {
                System.out.println ("Read problems");
                NodeList children2 = n1.getChildNodes ();
                for (int j = 0; j < children2.getLength (); j++) {
                    Node n2 = children2.item (j);
                    if (n2.getNodeName () == "problem") {
                        Problem p = new Problem ();
                        NodeList children3 = n2.getChildNodes ();
                        for (int k = 0; k < children3.getLength (); k++) {
                            Node n3 = children3.item (k);
                            if (n3.getNodeName () == "message") {
                                p.message = n3.getTextContent ();
                            }
                            if (n3.getNodeName () == "detail") {
                                p.detail = n3.getTextContent ();
                            }
                        }
                        m_problems.add (p);
                    }
                }
            }
        }
	}
	
    public void writeResult (String filepath) throws ExerciseException {
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
        try {
            DocumentBuilder docbuilder = factory.newDocumentBuilder ();
            Document doc = docbuilder.newDocument ();
            Element root = doc.createElement ("result");
            if (m_ok) {
                root.setAttribute ("passed", "yes");               
            }
            else {
                root.setAttribute ("passed", "no");               
            }
            doc.appendChild (root);
            
            if (m_problems.size () > 0) {
                System.out.println ("Write problems");
                Element problems = doc.createElement ("problems");
                root.appendChild (problems);
                for (Iterator<Problem> it = m_problems.iterator (); 
                        it.hasNext (); ) {
                    Problem p = it.next ();
                    Element pnode = doc.createElement ("problem");
                    problems.appendChild (pnode);
                    
                    Element e;
                    e = doc.createElement ("message");
                    e.setTextContent (p.message);
                    pnode.appendChild (e);
                    e = doc.createElement ("detail");
                    e.setTextContent (p.detail);
                    pnode.appendChild (e);
                }
            }
            
            TransformerFactory tfactory = TransformerFactory.newInstance (); 
            Transformer transformer = tfactory.newTransformer (); 

            Source src = new DOMSource (doc);
            
            FileOutputStream fos = new FileOutputStream (filepath);
            
            Result dest = new StreamResult (fos); 
            transformer.transform (src, dest); 
        }
        catch (Exception e) {
            System.out.println (e.getMessage ());
            e.printStackTrace ();
            throw new ExerciseException ("Could not write result: " + e.getMessage ());
           
        }
        
        System.out.println("Done...");
    }
}
