package no.ntnu.stud.frodesol.jexserver.eval;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import java.io.*;

public class TestTree {
	
	private TestNode m_rootnode = new TestNode ("Excercise Requirements");
	private Document m_document;
	
	public TestTree (Document jexdocument) {
		
		m_document = jexdocument;
	}
	
	public Document getJexDocument () {
		
		return m_document;
	}
	
	public TestNode getRootNode () {
		
		return m_rootnode;
	}

    public int getLastNodeNumber () {
        return getNodeCount () - 1;
    }
	
	public TestNode getNode (int number) {
	    
	    // Inefficient, should be replaced with a more efficient method
	    TestNode[] allnodes = getAllTestNodes ();
	    return allnodes[number];
	}
	
	public int getNodeCount () {
        // Inefficient, should be replaced with a more efficient method
    	return getAllTestNodes ().length;
	}
	
	public TestNode[] getAllTestNodes () {
		
		ArrayList list = new ArrayList ();
		appendTestsRecursivelyToList (getRootNode (), list);
		return (TestNode[]) list.toArray (new TestNode[0]);
	}
	
	private void appendTestsRecursivelyToList (TestNode node,
	        ArrayList list) {
		
		list.add (node);
		TestNode[] nodes = node.getChildren ();
		for (int i = 0; i < nodes.length; i++) {
			appendTestsRecursivelyToList (nodes[i], list);
		}
	}
	
	public int[] calculateResult () {
	    
	    int[] result = new int[2];
	    calculateResult (getRootNode (), result);
	    return result;
	}
	
	public void calculateResult (TestNode node, int[] result) {
	    
	    result[0] += node.getAchievedPoints ();
	    result[1] += node.getMaxPoints ();
	    
        TestNode[] nodes = node.getChildren ();
        for (int i = 0; i < nodes.length; i++) {
            calculateResult (nodes[i], result);
        }        
	}
	
    public void writeReport (String filepath, String compilerlogfile)
            throws IOException {
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder docbuilder = factory.newDocumentBuilder ();
            Document doc = docbuilder.newDocument ();
            Element root = doc.createElement ("report");
            doc.appendChild (root);
            Element testsnode = doc.createElement ("testresults");
            root.appendChild (testsnode);
            
            addReportTest (doc, testsnode, m_rootnode);
            
            try {
                Document logdoc = docbuilder.parse (compilerlogfile);
                Node importednode = doc.importNode (
                        logdoc.getDocumentElement (), true);
                root.appendChild (importednode);
            }
            catch (Exception e) {
                Node compnode = doc.createElement ("compiler");
                root.appendChild (compnode);
                
                Node errornode = doc.createElement ("error");
                compnode.appendChild (errornode);
                errornode.setTextContent ("Could not embed compiler log: " +
                        e.getMessage ());
            }
            TransformerFactory tfactory = TransformerFactory.newInstance (); 
            Transformer transformer = tfactory.newTransformer (); 

            Source src = new DOMSource (doc);
            
            FileOutputStream fos = null;
            try {
                fos = new FileOutputStream (filepath);
                Result dest = new StreamResult (fos); 
                transformer.transform (src, dest); 
            }
            finally {
                fos.close ();                
            }
        }
        catch (Exception e) {
            System.out.println (e.getMessage ());
            throw new IOException ("Could not write result: " + e.getMessage ());
        }
        
        System.out.println("Done...");
	}
    
    
    private void addReportTest (Document doc, 
            Element parent, TestNode node) {
        
        Element element = doc.createElement ("test");
        parent.appendChild (element);
        
        if (node.getMaxPoints () > 0) {
            element.setAttribute ("max", Integer.toString (
                    node.getMaxPoints ()));
            element.setAttribute ("achieved", Integer.toString (
                    node.getAchievedPoints ()));
        }
        Element e = doc.createElement ("name");
        element.appendChild (e);
        e.setTextContent (node.getName ());
        
        if (node.getProblems ().size () > 0) {
            System.out.println ("Write problems to report");
            Element problems = doc.createElement ("problems");
            element.appendChild (problems);
            for (Iterator<Problem> it = node.getProblems ().iterator (); 
                    it.hasNext (); ) {
                Problem p = it.next ();
                Element pnode = doc.createElement ("problem");
                problems.appendChild (pnode);
                
                e = doc.createElement ("message");
                e.setTextContent (p.message);
                pnode.appendChild (e);
                e = doc.createElement ("detail");
                e.setTextContent (p.detail);
                pnode.appendChild (e);
            }
        }
        
        TestNode[] nodes = node.getChildren ();
        for (int i = 0; i < nodes.length; i++) {
            addReportTest (doc, element, nodes[i]);
        }
    }
}
